#!/usr/bin/env python3
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2010-2023 German Aerospace Center (DLR) and others.
# This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v2.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v20.html
# SPDX-License-Identifier: EPL-2.0

# @file    runner.py
# @author  Vasilios Karagounis (CERTH HIT)
# @date    2025-03-28
# @version $Id$

import os
import sys
import optparse

try:
    sys.path.append(os.path.join(os.environ.get("SUMO_HOME"), "tools"))
    from sumolib import checkBinary  # noqa
except ImportError:
    print("WARNING: please declare environment variable 'SUMO_HOME' as the root directory of your sumo installation (it should contain folders 'bin', 'tools' and 'docs')")
    pass

try:
    import traci # type: ignore
except ImportError:
    print("WARNING: TRACI module not found")
    pass

step = 0
options = []

#-------------------------------------------------------------------------------------------
#
#-------------------------------------------------------------------------------------------
def run():

    global step

    stepsTillRead = 100
    stepsCounter = 0

    edge = "AB2A"
    
    radius = 1

    # use this radious for accurate results
    #radius = int(traci.lane.getLength(edge + "_0") / 2) + 1


    traci.edge.subscribeContext(edge, 
                                traci.constants.CMD_GET_VEHICLE_VARIABLE, 
                                radius, 
                                [ traci.constants.VAR_SPEED, traci.constants.VAR_LANE_ID ])
    
    while traci.simulation.getMinExpectedNumber() > 0:
    
        if stepsCounter >= stepsTillRead:
            stepsCounter = 0
            
            data = traci.edge.getContextSubscriptionResults(edge)
            traciTime = traci.simulation.getTime()
            print("time :", traciTime, "vehicles counted :", len(data))

        traci.simulationStep()

        step += 1
        stepsCounter+=1

#---------------------------------------------------------------------------------------------------------------------------------------------
def readOptions():

    global options

    optParser = optparse.OptionParser()

    optParser.add_option("-c", dest = "sumocfg", help = "sumo config file")
    optParser.add_option("--gui", action = "store_true", dest = "gui", default = False, help = "run with gui")

    options, _ = optParser.parse_args()

#---------------------------------------------------------------------------------------------------------------------------------------------
# this is the main entry point of this script
#---------------------------------------------------------------------------------------------------------------------------------------------
if __name__ == "__main__":
  
    readOptions()

    if  options.gui:
        sumoBinary = checkBinary('sumo-gui')
    else:
        sumoBinary = checkBinary('sumo')

    sumo_args = [sumoBinary, "-c", options.sumocfg]

    try:
        traci.start(sumo_args)

        while traci.simulation.getMinExpectedNumber() == 0:
            traci.simulationStep()
            step += 1

        run()

        traci.close()
    
    except traci.TraCIException as e1:
        print("Exception:", e1)
        pass
    except traci.FatalTraCIError as e2:
        print("Fatal exception:", e2)
        pass        

    
    sys.stdout.flush()
   
